# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

"""
Test that twisted scripts can be invoked as modules.
"""

from __future__ import division, absolute_import

import sys

from twisted.application.twist._options import TwistOptions
from twisted.scripts import trial
from twisted.internet import defer, reactor
from twisted.python.compat import NativeStringIO as StringIO
from twisted.test.test_process import Accumulator
from twisted.trial.unittest import TestCase


class MainTests(TestCase):
    """Test that twisted scripts can be invoked as modules."""
    # this test just does not work correctly on Gentoo
    # the output has '__main__.py' instead of 'trial'
    # I have only been able to get 2.7 working correctly
    # with replacing the value with what is expected.
    def test_twisted(self):
        """Invoking python -m twisted should execute twist."""
        cmd = sys.executable
        p = Accumulator()
        d = p.endedDeferred = defer.Deferred()
        reactor.spawnProcess(p, cmd, [cmd, '-m', 'twisted', '--help'], env=None)
        p.transport.closeStdin()

        # Fix up our sys args to match the command we issued
        from twisted import __main__
        self.patch(sys, 'argv', [__main__.__file__, '--help'])

        def processEnded(ign):
            f = p.outF
            # Some systems may return __main__.py instead of the command name expected
            output = f.getvalue().replace(b'\r\n', b'\n').replace(b"__main__.py", b"trial")

            options = TwistOptions()
            message = '{}\n'.format(options).encode('utf-8')
            # NOTE: python may return the  options in a different order
            # than is output via the command --help option
            # so we must break up the text and compare that lines
            # are not missing or extra from what is expected
            a = output.split(b'\n')
            b = message.split(b'\n')
            extras = []
            missing = []
            equal_len = (len(a) == len(b))
            for i in a:
                if i not in b:
                    extras.append(i)
            for i in b:
                if i not in a:
                    missing.append(i)

            self.assertTrue(equal_len,
                msg="Usage reported a different number of lines than expected")
            self.assertTrue(extras == [],
                msg="Usage returned these extra lines not expected: %s"
                    % '\n'.join(extras)
            )
            self.assertTrue(missing == [],
                msg="Usage was missing these expected lines: %s"
                    % '\n'.join(missing)
            )
        return d.addCallback(processEnded)

    def test_trial(self):
        """Invoking python -m twisted.trial should execute trial."""
        cmd = sys.executable
        p = Accumulator()
        d = p.endedDeferred = defer.Deferred()
        reactor.spawnProcess(p, cmd, [cmd, '-m', 'twisted.trial', '--help'], env=None)
        p.transport.closeStdin()

        # Fix up our sys args to match the command we issued
        from twisted.trial import __main__
        self.patch(sys, 'argv', [__main__.__file__, '--help'])

        def processEnded(ign):
            f = p.outF
            output = f.getvalue().replace(b'\r\n', b'\n')
            
            options = trial.Options()
            message = '{}\n'.format(options).encode('utf-8')
            self.assertEqual(output, message)
        return d.addCallback(processEnded)

    def test_twisted_import(self):
        """Importing twisted.__main__ does not execute twist."""
        output = StringIO()
        monkey = self.patch(sys, 'stdout', output)

        import twisted.__main__
        self.assertTrue(twisted.__main__)  # Appease pyflakes

        monkey.restore()
        self.assertEqual(output.getvalue(), "")
